local addon_name, addon_shared = ...;

local _G = getfenv(0);

local LibStub = LibStub;
local Addon = LibStub("AceAddon-3.0"):NewAddon(addon_name, "AceEvent-3.0");
_G[addon_name] = Addon;

_G["BINDING_HEADER_PORTALMASTER"] = "PortalMaster";
_G["BINDING_NAME_TOGGLE_PORTALMASTER"] = "Toggle PortalMaster";

local MagePortSpells = {
	Alliance	= {
		Teleport = {
			3561, 	-- Stormwind
			3562, 	-- Ironforge
			3565, 	-- Darnassus
			32271,	-- Exodar
			49359,	-- Theramore
			
			33690,	-- Shattrath
			53140,	-- Dalaran
			120145,	-- Ancient Dalaran
			
			88342,	-- Tol Barad
			
			132621,	-- Vale of Eternal Blossoms
			176248,	-- Stormshield
		},
		
		Portal 	= {
			10059, 	-- Stormwind
			11416, 	-- Ironforge
			11419, 	-- Darnassus
			32266,	-- Exodar
			49360,	-- Theramore
			
			33691,	-- Shattrath
			53142,	-- Dalaran
			120146,	-- Ancient Dalaran
			
			88345,	-- Tol Barad
			
			132620,	-- Vale of Eternal Blossoms
			176246,	-- Stormshield
		},
	},
	
	Horde 		= {
		Teleport = {
			3567, 	-- Orgrimmar
			3566, 	-- Thunder Bluff
			3563, 	-- Undercity
			32272,	-- Silvermoon
			49358,	-- Stonard
			
			35715,	-- Shattrath
			53140,	-- Dalaran
			120145,	-- Ancient Dalaran
			
			88344,	-- Tol Barad
			
			132627,	-- Vale of Eternal Blossoms
			176242, -- Warspear
		},
		
		Portal 	= {
			11417, 	-- Orgrimmar
			11420, 	-- Thunder Bluff
			11418, 	-- Undercity
			32267,	-- Silvermoon
			49361,	-- Stonard
			
			35717,	-- Shattrath
			53142,	-- Dalaran
			120146,	-- Ancient Dalaran
			
			88346,	-- Tol Barad
			
			132626,	-- Vale of Eternal Blossoms
			176244,	-- Warspear
		},
	},
};

local FACTION_NAME = UnitFactionGroup("player");

local MESSAGE_PATTERN = "|cffe8608fPortalMaster|r %s";
function Addon:AddMessage(pattern, ...)
	DEFAULT_CHAT_FRAME:AddMessage(MESSAGE_PATTERN:format(string.format(pattern, ...)));
end

function Addon:OnInitialize()
	
end

local _, PLAYER_CLASS = UnitClass("player");

function Addon:OnEnable()
	if(PLAYER_CLASS == "MAGE") then
		Addon:RegisterEvent("PLAYER_REGEN_DISABLED");
	end
end

function Addon:PLAYER_REGEN_DISABLED()
	Addon:CloseFrame();
end

function Addon:ResetFrame()
	PortalMasterFrameSearch:SetText("");
end

function Addon:ToggleFrame()
	if(InCombatLockdown()) then return end
	if(PLAYER_CLASS ~= "MAGE") then Addon:AddMessage("Not a mage"); return end
	
	if(not PortalMasterFrame:IsShown()) then
		Addon:OpenFrame();
	else
		Addon:CloseFrame();
	end
end

function Addon:OpenFrame()
	if(PLAYER_CLASS ~= "MAGE") then return end
	
	if(PortalMasterFrame:IsShown()) then return end
	
	-- if(not Addon.CurrentBinding) then
		-- Addon.CurrentBinding = GetBindingAction("ENTER");
		-- SetBinding("ENTER", "CLICK PortalMasterFrameSpellButton:LeftButton");
		-- SetOverrideBindingClick(PortalMasterFrameSearch, true, "ENTER", "PortalMasterFrameSpellButton", "LeftButton");
	-- end
	
	
	Addon:ResetFrame();
	PortalMasterFrame:Show();
	
	PortalMasterFrameSearch:Show();
	PortalMasterFrameSearch:SetFocus();
	
	PortalMasterFrameSpellConfirm:Hide();
end

function Addon:CloseFrame()
	if(Addon.CurrentBinding) then
		SetBinding("ENTER", Addon.CurrentBinding);
		Addon.CurrentBinding = nil;
	end
	
	PortalMasterFrame:Hide();
end

function PortalMaster_OnEditFocusLost()
	-- Addon:CloseFrame()
end

function PortalMaster_OnEnterPressed(self)
	if(not Addon.CurrentBinding) then
		Addon.CurrentBinding = GetBindingAction("ENTER");
		SetBinding("ENTER", "CLICK PortalMasterFrameSpellButton:LeftButton");
	end
	
	PortalMasterFrameSearch:Hide();
	PortalMasterFrameSpellConfirm:Show();
end

function PortalMaster_OnTextChanged(self)
	SearchBoxTemplate_OnTextChanged(self)
	
	local category = "Teleport";
	local searchText = strtrim(strlower(self:GetText()));
	
	local first, rest = strsplit(" ", searchText, 2);
	if(strtrim(first) == "p" or strtrim(first) == "portal") then
		category = "Portal";
		searchText = rest or "";
	end
	
	if(strlen(searchText) > 0) then
		local tokens = { strsplit(" ", searchText) };
		
		for index, spellID in ipairs(MagePortSpells[FACTION_NAME][category]) do
			if(IsSpellKnown(spellID)) then
				local spellName, _, icon = GetSpellInfo(spellID);
				
				local searchSpellName = gsub(spellName, "Teleport: ", "");
				searchSpellName = gsub(searchSpellName, "Portal: ", "");
				
				local spellFound = true;
				for _, token in ipairs(tokens) do
					spellFound = spellFound and strmatch(strlower(searchSpellName), token);
				end
				
				if(spellFound) then
					PortalMasterFrameSpellName:SetText(spellName);
					
					PortalMasterFrameSpellButton.icon:SetTexture(icon);
					PortalMasterFrameSpellButton.iconBorder:SetVertexColor(0.1, 0.1, 0.1);
					
					PortalMasterFrameSpellButton:SetAttribute("type", "spell");
					PortalMasterFrameSpellButton:SetAttribute("spell", spellName);
					
					PortalMasterFrameSpellButton:Show();
					
					break;
				else
					PortalMasterFrameSpellName:SetText("No Result");
					PortalMasterFrameSpellButton:Hide();
				end
			end
		end
	else
		PortalMasterFrameSpellName:SetText("Enter Spell Name");
		PortalMasterFrameSpellButton:Hide();
	end
end

function PortalMaster_OnEscapePressed(self)
	self:ClearFocus();
	Addon:CloseFrame();
end

function PortalMaster_CloseFrame()
	Addon:CloseFrame();
end

function Addon:OnDisable()
		
end
